"""Test the Sentinel class."""

from enve.parser import UNSET, Sentinel


def test_sentinel_repr() -> None:
    """Test the __repr__ method of the Sentinel class."""
    assert repr(UNSET) == "Sentinel(UNSET)"


def test_sentinel_reduce() -> None:
    """Test the __reduce__ method of the Sentinel class."""
    assert UNSET.__reduce__() == (Sentinel, ("UNSET", "enve.parser"))


def test_sentinel_registry() -> None:
    """Test the registry of the Sentinel class."""
    unset = Sentinel("UNSET")
    assert unset is UNSET

    new_sentinel = Sentinel("NEW_SENTINEL")
    assert new_sentinel is not UNSET



"""Test that typing works as expected."""

import re
import shlex
import subprocess as sp
import sys

from pathlib import Path
from tempfile import TemporaryDirectory

import pytest


def get_mypy_revelations(code: str) -> list[str]:
    """Run mypy and return the revealed type."""
    with TemporaryDirectory() as tmpdir:
        code_file = Path(tmpdir) / "test.py"
        with open(code_file, "w", encoding="utf-8") as f:
            f.write(code)

        cache_dir = Path(tmpdir) / ".mypy_cache"

        cmd = [
            sys.executable,
            "-m",
            "mypy",
            "--cache-dir",
            str(cache_dir),
            "--no-color-output",
            "--no-error-summary",
            "--no-incremental",
            str(code_file),
        ]
        cmd_str = shlex.join(cmd)

        pipe = sp.run(
            cmd_str if sys.platform == "win32" else cmd,
            capture_output=True,
            check=False,
            text=True,
        )

    pattern = re.compile(r"Revealed type is \"(.+)\"$")
    output = pipe.stdout
    lines = output.splitlines()

    matches = (pattern.search(line) for line in lines)
    revelations = [match.group(1).strip() for match in matches if match]

    return revelations


TYPING_TO_REVEAL = {
    # str
    "get('a')": "builtins.str",
    "get('a', default=None)": "Union[builtins.str, None]",
    "get('a', dtype=str)": "builtins.str",
    "get('a', default=None, dtype=str)": "Union[builtins.str, None]",
    # int
    "get('a', dtype=int)": "builtins.int",
    "get('a', default=None, dtype=int)": "Union[builtins.int, None]",
    # float
    "get('a', dtype=float)": "builtins.float",
    "get('a', default=None, dtype=float)": "Union[builtins.float, None]",
    # bool
    "get('a', dtype=bool)": "builtins.bool",
    "get('a', default=None, dtype=bool)": "Union[builtins.bool, None]",
    # bytes
    "get('a', dtype=bytes)": "builtins.bytes",
    "get('a', default=None, dtype=bytes)": "Union[builtins.bytes, None]",
    # list + None
    "get('a', dtype=list)": "builtins.list[builtins.str]",
    "get('a', default=None, dtype=list)": "Union[builtins.list[builtins.str], None]",
    # list + str
    "get('a', dtype=list, item_dtype=str)": "builtins.list[builtins.str]",
    "get('a', default=None, dtype=list, item_dtype=str)": "Union[builtins.list[builtins.str], None]",  # noqa: E501
    # list + int
    "get('a', dtype=list, item_dtype=int)": "builtins.list[builtins.int]",
    "get('a', default=None, dtype=list, item_dtype=int)": "Union[builtins.list[builtins.int], None]",  # noqa: E501
    # list + float
    "get('a', dtype=list, item_dtype=float)": "builtins.list[builtins.float]",
    "get('a', default=None, dtype=list, item_dtype=float)": "Union[builtins.list[builtins.float], None]",  # noqa: E501
    # list + bool
    "get('a', dtype=list, item_dtype=bool)": "builtins.list[builtins.bool]",
    "get('a', default=None, dtype=list, item_dtype=bool)": "Union[builtins.list[builtins.bool], None]",  # noqa: E501
    # list + bytes
    "get('a', dtype=list, item_dtype=bytes)": "builtins.list[builtins.bytes]",
    "get('a', default=None, dtype=list, item_dtype=bytes)": "Union[builtins.list[builtins.bytes], None]",  # noqa: E501
    # tuple + None
    "get('a', dtype=tuple)": "builtins.tuple[builtins.str, ...]",
    "get('a', default=None, dtype=tuple)": "Union[builtins.tuple[builtins.str, ...], None]",
    # tuple + str
    "get('a', dtype=tuple, item_dtype=str)": "builtins.tuple[builtins.str, ...]",
    "get('a', default=None, dtype=tuple, item_dtype=str)": "Union[builtins.tuple[builtins.str, ...], None]",  # noqa: E501
    # tuple + int
    "get('a', dtype=tuple, item_dtype=int)": "builtins.tuple[builtins.int, ...]",
    "get('a', default=None, dtype=tuple, item_dtype=int)": "Union[builtins.tuple[builtins.int, ...], None]",  # noqa: E501
    # tuple + float
    "get('a', dtype=tuple, item_dtype=float)": "builtins.tuple[builtins.float, ...]",
    "get('a', default=None, dtype=tuple, item_dtype=float)": "Union[builtins.tuple[builtins.float, ...], None]",  # noqa: E501
    # tuple + bool
    "get('a', dtype=tuple, item_dtype=bool)": "builtins.tuple[builtins.bool, ...]",
    "get('a', default=None, dtype=tuple, item_dtype=bool)": "Union[builtins.tuple[builtins.bool, ...], None]",  # noqa: E501
    # tuple + bytes
    "get('a', dtype=tuple, item_dtype=bytes)": "builtins.tuple[builtins.bytes, ...]",
    "get('a', default=None, dtype=tuple, item_dtype=bytes)": "Union[builtins.tuple[builtins.bytes, ...], None]",  # noqa: E501
}


@pytest.fixture(scope="module")
def mypy_revelations() -> dict[str, str]:
    """Fixture to run mypy and return the revealed type alongside the code."""
    setup = "from enve import get"
    statements = [f"reveal_type({s})" for s in TYPING_TO_REVEAL]
    code = "\n".join([setup, *statements])

    revelations = get_mypy_revelations(code)
    if len(revelations) != len(TYPING_TO_REVEAL):
        err_msg = f"Expected {len(TYPING_TO_REVEAL)} revelations, but got {len(revelations)}"
        raise RuntimeError(err_msg)

    results = dict(zip(TYPING_TO_REVEAL, revelations, strict=True))
    return results


@pytest.mark.parametrize("code", list(TYPING_TO_REVEAL.keys()))
def test_mypy_revelations(code: str, mypy_revelations: dict[str, str]) -> None:
    """Test that the revealed type of get() is as expected."""
    expected_type = TYPING_TO_REVEAL[code]
    revealed_type = mypy_revelations[code]

    err_msg = f"{code}: Expected '{expected_type}', got '{revealed_type}'"
    assert revealed_type == expected_type, err_msg



# MIT License

# Copyright (c) 2025 Aurélien Chick

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
"""Test that getting the version works."""

from importlib.metadata import PackageNotFoundError

import pytest

from enve import _get_module_version


def test_get_version_without_installation(monkeypatch: pytest.MonkeyPatch) -> None:
    """Test that a version can be returned without installation."""
    monkeypatch.setattr(
        "enve._get_version", lambda _: (_ for _ in ()).throw(PackageNotFoundError())
    )
    assert _get_module_version() == "0.0.0"


def test_get_version_with_pinned_version(monkeypatch: pytest.MonkeyPatch) -> None:
    """Test that getting the version works."""
    monkeypatch.setattr("enve._get_version", lambda _: "X.Y.Z")
    assert _get_module_version() == "X.Y.Z"



"""Test the CLI entrypoint for the enve library."""

import json
import os
import subprocess as sp
import sys

import pytest

from enve.cli import _get_docker_secret_value


CDIR = os.path.dirname(os.path.abspath(__file__))
PROJ_DIR = os.path.dirname(CDIR)
BASE_CMD = [
    sys.executable,
    "-m",
    "coverage",
    "run",
    os.path.join(PROJ_DIR, "src", "enve", "cli.py"),
]


def test_main_with_envvar() -> None:
    """Test the main function using subprocess."""
    env_var = "TEST_ENV_VAR"
    env_value = "test_value"
    env = {env_var: env_value, "COVERAGE_PROCESS_START": "1"}

    pipe = sp.run(
        [*BASE_CMD, env_var],
        capture_output=True,
        check=False,
        env=env,
        text=True,
    )

    assert pipe.returncode == 0, f"Return code: {pipe.returncode} (!= 0): {pipe.stderr}"
    assert env_value == pipe.stdout


def test_main_with_missing_envvar() -> None:
    """Test the main function with a missing environment variable."""
    env_var = "TEST_ENV_VAR"
    env = {"COVERAGE_PROCESS_START": "1"}

    pipe = sp.run(
        [*BASE_CMD, env_var],
        capture_output=True,
        check=False,
        env=env,
        text=True,
    )

    assert pipe.returncode == 1, f"Return code: {pipe.returncode} (!= 0): {pipe.stderr}"
    expected_msg = f"Environment variable '{env_var}' is not set and no default or default_factory is provided."  # noqa: E501
    assert expected_msg in pipe.stderr


def test_main_with_piped_in_input() -> None:
    """Test the main function with piped input."""
    env_var = "TEST_ENV_VAR"
    env_value = "test_value"
    env = {env_var: env_value, "COVERAGE_PROCESS_START": "1"}

    pipe = sp.run(
        [*BASE_CMD],
        input=env_var,
        capture_output=True,
        check=False,
        env=env,
        text=True,
    )

    assert pipe.returncode == 0, f"Return code: {pipe.returncode} (!= 0): {pipe.stderr}"
    assert env_value == pipe.stdout


def test_main_with_piped_output() -> None:
    """Test the main function with piped output."""
    env_var = "TEST_ENV_VAR"
    env_value = "test_value"
    env = {env_var: env_value, "COVERAGE_PROCESS_START": "1"}

    cmd = "type" if sys.platform == "win32" else "cat"

    output = sp.check_output(
        " ".join([*BASE_CMD, env_var, "|", cmd]),
        env=env,
        shell=True,
        text=True,
    )
    assert env_value == output


@pytest.mark.parametrize("args", [[], ["--null"]])
def test_main_print_all_envvars(args: list[str]) -> None:
    """Test the main function printing of all envvars."""
    env = {"COVERAGE_PROCESS_START": "1", "FOOBAR": "1"}

    pipe = sp.run(
        [*BASE_CMD, *args],
        capture_output=True,
        check=False,
        env=env,
        text=True,
    )

    pipe_stdout = pipe.stdout.strip()
    assert pipe.returncode == 0
    for env_var, env_val in env.items():
        assert f"{env_var}={env_val}" in pipe_stdout


def test_main_print_all_envvars_to_json() -> None:
    """Test the main function's print of all envvars in JSON format."""
    env = {"COVERAGE_PROCESS_START": "1", "FOOBAR": "1"}

    pipe = sp.run(
        [*BASE_CMD, "--json"],
        capture_output=True,
        check=False,
        env=env,
        text=True,
    )

    pipe_stdout = pipe.stdout.strip()
    assert pipe.returncode == 0
    returned_json = json.loads(pipe_stdout)
    assert isinstance(returned_json, dict)
    for env_var, env_val in env.items():
        assert env_var in returned_json
        assert returned_json[env_var] == env_val


@pytest.mark.parametrize(("value", "expected"), [(None, False), ("", True), ("FOOBAR", "FOOBAR")])
def test_get_docker_secret_value(value: str, expected: bool | str) -> None:
    """Test the _get_docker_secret_value function."""
    result = _get_docker_secret_value(value)
    assert result == expected, f"Expected {expected}, got {result}"



# MIT License

# Copyright (c) 2025 Aurélien Chick

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
"""Parse environment variables into Python objects."""

import os
import warnings

from collections.abc import Callable
from pathlib import Path
from threading import Lock
from typing import TypeAlias, TypeVar, cast, overload


class EnvError(KeyError, ValueError):
    """
    An environment variable parsing error.

    This class inherits from `KeyError` and `ValueError` in an attempt
    to be catchable by more traditional exception handling code.
    """

    def __init__(self, message: str) -> None:
        super().__init__(message)
        self.message = message

    def __str__(self) -> str:
        return self.message


_lock = Lock()
_registry: dict[str, "Sentinel"] = {}


class Sentinel:
    """
    Create a unique sentinel object.

    This is a stripped down version of the `Sentinel` class defined here:
    https://github.com/taleinat/python-stdlib-sentinels/tree/main
    """

    _name: str
    _module_name: str

    def __new__(cls, name: str) -> "Sentinel":
        """Create a new sentinel object with a unique name."""
        name = str(name)
        module_name = "enve.parser"

        registry_key = f"{module_name}-{name}"
        sentinel = _registry.get(registry_key)
        if sentinel is not None:
            return sentinel

        sentinel = super().__new__(cls)
        sentinel._name = name
        sentinel._module_name = str(module_name)
        with _lock:
            return _registry.setdefault(registry_key, sentinel)

    def __repr__(self) -> str:
        return f"{self.__class__.__name__}({self._name})"

    def __reduce__(self) -> tuple[type, tuple[str, str]]:
        return (self.__class__, (self._name, self._module_name))


UNSET = Sentinel("UNSET")
"""A sentinel value to indicate that a value is unset."""
EnvType: TypeAlias = bool | bytes | float | int | str | list | tuple
ItemType = TypeVar("ItemType", bound=bool | bytes | float | int | str)


def _parse_bool(envvar: str, value: str) -> bool:
    truthy_values = ("1", "on", "t", "true", "y", "yes")
    falsy_values = ("", "0", "off", "f", "false", "n", "no")
    if value.lower() in truthy_values:
        return True
    if value.lower() in falsy_values:
        return False

    err_msg = (
        f"Environment variable '{envvar}' is not a valid boolean value. "
        f"Expected one of {truthy_values + falsy_values}."
    )
    raise EnvError(err_msg)


def _parse_float(envvar: str, value: str) -> float:
    try:
        float_value = float(value)
    except (TypeError, ValueError):
        err_msg = f"Environment variable '{envvar}' is not a valid float: '{value}'"
        raise EnvError(err_msg) from None
    return float_value


def _parse_int(envvar: str, value: str) -> int:
    try:
        int_value = int(value)
    except (TypeError, ValueError):
        err_msg = f"Environment variable '{envvar}' is not a valid integer: '{value}'"
        raise EnvError(err_msg) from None
    return int_value


def _parse_list(envvar: str, values: list[str], item_dtype: type[ItemType]) -> list:
    accepted_item_dtypes = (bool, bytes, float, int, str)
    if item_dtype not in accepted_item_dtypes:
        err_msg = (
            f"Invalid item type '{item_dtype}' for '{envvar}'."
            f" Expected one of {accepted_item_dtypes}."
        )
        raise TypeError(err_msg)

    if item_dtype is bool:
        return [_parse_bool(f"{envvar}[{i}]", v) for i, v in enumerate(values)]
    elif item_dtype is bytes:
        return [v.encode("utf-8") for v in values]
    elif item_dtype is float:
        return [_parse_float(f"{envvar}[{i}]", v) for i, v in enumerate(values)]
    elif item_dtype is int:
        return [_parse_int(f"{envvar}[{i}]", v) for i, v in enumerate(values)]
    else:  # item_dtype is str
        return values


def _get_envvar_value(envvar: str, *, docker_secret: bool | str = False) -> str | object:
    """Load an environment variable from a .env file."""
    value = os.environ.get(envvar, UNSET)
    if value is not UNSET:
        return value

    # `value` is UNSET, so we check if the envvar is a docker secret.
    if docker_secret is True:
        secrets_paths = (
            Path(f"/run/secrets/{envvar}"),
            Path(f"/run/secrets/{envvar.lower()}"),
            Path(f"C:\\ProgramData\\Docker\\secrets\\{envvar}"),
            Path(f"C:\\ProgramData\\Docker\\secrets\\{envvar.lower()}"),
        )
        for path in secrets_paths:
            if path.exists():
                value = path.read_text()
                break
    elif isinstance(docker_secret, str):
        paths = (
            Path(f"/run/secrets/{docker_secret}"),
            Path(f"C:\\ProgramData\\Docker\\secrets\\{envvar}"),
        )
        for path in paths:
            if path.exists():
                value = path.read_text()
                break

    return value


@overload
def get(
    envvar: str,
    *,
    default: str | Sentinel = ...,
    default_factory: Callable[[], EnvType] | None = ...,
    dtype: None = None,
    deprecated: bool = ...,
    deprecated_msg: str = ...,
    docker_secret: bool | str = ...,
    item_dtype: type[ItemType] | None = ...,
    item_sep: str = ...,
) -> str: ...


@overload
def get(
    envvar: str,
    *,
    default: None = None,
    default_factory: Callable[[], EnvType] | None = ...,
    dtype: None = None,
    deprecated: bool = ...,
    deprecated_msg: str = ...,
    docker_secret: bool | str = ...,
    item_dtype: type[ItemType] | None = ...,
    item_sep: str = ...,
) -> str | None: ...


@overload
def get(
    envvar: str,
    *,
    default: bool | Sentinel = ...,
    default_factory: Callable[[], EnvType] | None = ...,
    dtype: type[bool] = bool,
    deprecated: bool = ...,
    deprecated_msg: str = ...,
    docker_secret: bool | str = ...,
    item_dtype: type[ItemType] | None = ...,
    item_sep: str = ...,
) -> bool: ...


@overload
def get(
    envvar: str,
    *,
    default: None = None,
    default_factory: Callable[[], EnvType] | None = ...,
    dtype: type[bool] = bool,
    deprecated: bool = ...,
    deprecated_msg: str = ...,
    docker_secret: bool | str = ...,
    item_dtype: type[ItemType] | None = ...,
    item_sep: str = ...,
) -> bool | None: ...


@overload
def get(
    envvar: str,
    *,
    default: int | Sentinel = ...,
    default_factory: Callable[[], EnvType] | None = ...,
    dtype: type[int] = int,
    deprecated: bool = ...,
    deprecated_msg: str = ...,
    docker_secret: bool | str = ...,
    item_dtype: type[ItemType] | None = ...,
    item_sep: str = ...,
) -> int: ...


@overload
def get(
    envvar: str,
    *,
    default: None = None,
    default_factory: Callable[[], EnvType] | None = ...,
    dtype: type[int] = int,
    deprecated: bool = ...,
    deprecated_msg: str = ...,
    item_dtype: type[ItemType] | None = ...,
    item_sep: str = ...,
) -> int | None: ...


@overload
def get(
    envvar: str,
    *,
    default: float | Sentinel = ...,
    default_factory: Callable[[], EnvType] | None = ...,
    dtype: type[float] = float,
    deprecated: bool = ...,
    deprecated_msg: str = ...,
    docker_secret: bool | str = ...,
    item_dtype: type[ItemType] | None = ...,
    item_sep: str = ...,
) -> float: ...


@overload
def get(
    envvar: str,
    *,
    default: None = None,
    default_factory: Callable[[], EnvType] | None = ...,
    dtype: type[float] = float,
    deprecated: bool = ...,
    deprecated_msg: str = ...,
    docker_secret: bool | str = ...,
    item_dtype: type[ItemType] | None = ...,
    item_sep: str = ...,
) -> float | None: ...


@overload
def get(
    envvar: str,
    *,
    default: str | Sentinel = ...,
    default_factory: Callable[[], EnvType] | None = ...,
    dtype: type[str] = str,
    deprecated: bool = ...,
    deprecated_msg: str = ...,
    docker_secret: bool | str = ...,
    item_dtype: type[ItemType] | None = ...,
    item_sep: str = ...,
) -> str: ...


@overload
def get(
    envvar: str,
    *,
    default: None = None,
    default_factory: Callable[[], EnvType] | None = ...,
    dtype: type[str],
    deprecated: bool = ...,
    deprecated_msg: str = ...,
    docker_secret: bool | str = ...,
    item_dtype: type[ItemType] | None = ...,
    item_sep: str = ...,
) -> str | None: ...


@overload
def get(
    envvar: str,
    *,
    default: bytes | Sentinel = ...,
    default_factory: Callable[[], EnvType] | None = ...,
    dtype: type[bytes] = bytes,
    deprecated: bool = ...,
    deprecated_msg: str = ...,
    docker_secret: bool | str = ...,
    item_dtype: type[ItemType] | None = ...,
    item_sep: str = ...,
) -> bytes: ...


@overload
def get(
    envvar: str,
    *,
    default: None = None,
    default_factory: Callable[[], EnvType] | None = ...,
    dtype: type[bytes],
    deprecated: bool = ...,
    deprecated_msg: str = ...,
    docker_secret: bool | str = ...,
    item_dtype: type[ItemType] | None = ...,
    item_sep: str = ...,
) -> bytes | None: ...


@overload
def get(
    envvar: str,
    *,
    default: list | Sentinel = ...,
    default_factory: Callable[[], EnvType] | None = ...,
    dtype: type[list] = list,
    deprecated: bool = ...,
    deprecated_msg: str = ...,
    docker_secret: bool | str = ...,
    item_dtype: None = ...,
    item_sep: str = ...,
) -> list[str]: ...


@overload
def get(
    envvar: str,
    *,
    default: list | Sentinel = ...,
    default_factory: Callable[[], EnvType] | None = ...,
    dtype: type[list] = list,
    deprecated: bool = ...,
    deprecated_msg: str = ...,
    docker_secret: bool | str = ...,
    item_dtype: type[ItemType] = ...,
    item_sep: str = ...,
) -> list[ItemType]: ...


@overload
def get(
    envvar: str,
    *,
    default: None = None,
    default_factory: Callable[[], EnvType] | None = ...,
    dtype: type[list],
    deprecated: bool = ...,
    deprecated_msg: str = ...,
    docker_secret: bool | str = ...,
    item_dtype: None = ...,
    item_sep: str = ...,
) -> list[str] | None: ...


@overload
def get(
    envvar: str,
    *,
    default: None = None,
    default_factory: Callable[[], EnvType] | None = ...,
    dtype: type[list],
    deprecated: bool = ...,
    deprecated_msg: str = ...,
    docker_secret: bool | str = ...,
    item_dtype: type[ItemType] = ...,
    item_sep: str = ...,
) -> list[ItemType] | None: ...


@overload
def get(
    envvar: str,
    *,
    default: tuple | Sentinel = ...,
    default_factory: Callable[[], EnvType] | None = ...,
    dtype: type[tuple] = tuple,
    deprecated: bool = ...,
    deprecated_msg: str = ...,
    docker_secret: bool | str = ...,
    item_dtype: None = ...,
    item_sep: str = ...,
) -> tuple[str, ...]: ...


@overload
def get(
    envvar: str,
    *,
    default: tuple | Sentinel = ...,
    default_factory: Callable[[], EnvType] | None = ...,
    dtype: type[tuple] = tuple,
    deprecated: bool = ...,
    deprecated_msg: str = ...,
    docker_secret: bool | str = ...,
    item_dtype: type[ItemType] = ...,
    item_sep: str = ...,
) -> tuple[ItemType, ...]: ...


@overload
def get(
    envvar: str,
    *,
    default: None = None,
    default_factory: Callable[[], EnvType] | None = ...,
    dtype: type[tuple],
    deprecated: bool = ...,
    deprecated_msg: str = ...,
    docker_secret: bool | str = ...,
    item_dtype: None = ...,
    item_sep: str = ...,
) -> tuple[str, ...] | None: ...


@overload
def get(
    envvar: str,
    *,
    default: None = None,
    default_factory: Callable[[], EnvType] | None = ...,
    dtype: type[tuple],
    deprecated: bool = ...,
    deprecated_msg: str = ...,
    docker_secret: bool | str = ...,
    item_dtype: type[ItemType] = ...,
    item_sep: str = ...,
) -> tuple[ItemType, ...] | None: ...


def get(
    envvar: str,
    *,
    default: EnvType | Sentinel | None = UNSET,
    default_factory: Callable[[], EnvType] | None = None,
    dtype: type[EnvType] | None = None,
    deprecated: bool = False,
    deprecated_msg: str = "",
    docker_secret: bool | str = False,
    item_sep: str = ",",
    item_dtype: type[ItemType] | None = None,
) -> EnvType | None:
    """
    Parse an environment variable into a bool, bytes, float, int, str, list or a tuple.

    Rather than using `os.environ.get` or `os.getenv`, this function
    provides a convenient way to parse environment variables into a
    specific data type with type checking and conversion support.

    Valid boolean environment variable values are (case insensitive):
    - "1", "on", "t", "true", "y", "yes" for True
    - "", "0", "off", "f", "false", "n", "no" for False

    The function also supports parsing lists of values, where the values
    are separated by a specified separator (default is ","). The list items
    can be converted to a specific data type using the `item_dtype` parameter.

    Parameters
    ----------
    envvar : str
        The name of the environment variable to parse.
    default : EnvType | Sentinel | None, default UNSET
        The default value to return if the environment variable is not set.
        If UNSET, a ValueError will be raised if the environment variable is not set.
    default_factory : Callable[[], EnvType] | None, default None
        A callable that returns the default value if the environment variable is not set.
        If None, this will be ignored. This cannot be used alongside `default`. This should
        be used instead of `default` when the expected return type is a list.
    dtype : type[bool | bytes | float | int | str | list | tuple] | None, default None
        The data type to convert the environment variable to. If None, the value
        will be returned as a string. Supported types are bool, bytes, float, int, str,
        list, and tuple.
    deprecated : bool, default False
        If True, and the environment variable is set, a DeprecationWarning will be raised.
    deprecated_msg : str, default ""
        A custom message to include in the DeprecationWarning if deprecated is True.
    docker_secret : bool | str, default False
        If True, the function will attempt to read the environment variable value from
        a Docker secret file. Attempted file names are `/run/secrets/{envvar}` and
        `/run/secrets/{envvar.lower()}`. If a string is provided, it will be used as
        the file name instead of the environment variable name. If False, the function
        will not check for Docker secrets.
    item_dtype : type[bool | bytes | float | int | str] | None, default None
        The data type to convert the items in the list to. If None, the items will
        be returned as strings. Supported types are bool, bytes, float, int, and str.
    item_sep : str, default ","
        The separator used to split the list of values in the environment variable.
        Only used if dtype is list. The default is ",".

    Returns
    -------
    bool |  bytes | float | int | str | list | tuple | None
        The parsed value of the environment variable, converted to the specified type.
        If the environment variable is not set and no default is provided, a ValueError
        will be raised. If the environment variable is set but cannot be converted to
        the specified type, a ValueError will be raised.

    Raises
    ------
    TypeError
        If the dtype is not one of (bool, bytes, float, int, str, list, tuple).
        If the default value is not of the specified dtype.
        If the item_dtype is not one of (bool, bytes, float, int, str).
    EnvError
        If the environment variable is not set and no default is provided.
        If both default and default_factory are provided.
        If the environment variable cannot be converted to the specified dtype.

    Notes
    -----
    Only "simple" data types are supported by this library as environment
    variables shouldn't be used to store complex data types.

    Examples
    --------
    Environment variable parsing can be done by importing `enve`:
    >>> import os
    >>> import enve

    Parsing an environment variable as a string:

    >>> os.environ["ENV_VAR"] = "test"
    >>> enve.get("ENV_VAR")
    'test'
    >>> enve.get("ENV_VAR", dtype=str)
    'test'

    Parsing an environment variable as a boolean:

    >>> os.environ["ENV_VAR"] = "true"
    >>> enve.get("ENV_VAR", dtype=bool)
    True

    Parsing an environment variable as bytes:

    >>> os.environ["ENV_VAR"] = "random_bytes"
    >>> enve.get("ENV_VAR", dtype=bytes)
    b'random_bytes'

    Parsing an environment variable as a float:

    >>> os.environ["ENV_VAR"] = "3.14"
    >>> enve.get("ENV_VAR", dtype=float)
    3.14

    Parsing an environment variable as an integer:

    >>> os.environ["ENV_VAR"] = "42"
    >>> enve.get("ENV_VAR", dtype=int)
    42

    Parsing an unset environment variable:

    >>> _ = os.environ.pop("ENV_VAR", None)
    >>> enve.get("ENV_VAR")
    Traceback (most recent call last):
        ...
    enve.parser.EnvError: Environment variable 'ENV_VAR' is not set and no default or default_factory is provided.

    Parsing an unset environment variable with a default value:

    >>> _ = os.environ.pop("ENV_VAR", None)
    >>> enve.get("ENV_VAR", default="default_value")
    'default_value'

    Parsing an environment variable as a list:

    >>> os.environ["ENV_VAR"] = "1,2,3"
    >>> enve.get("ENV_VAR", dtype=list)
    ['1', '2', '3']

    Parsing an environment variable as a list of integers:

    >>> os.environ["ENV_VAR"] = "1,2,3"
    >>> enve.get("ENV_VAR", dtype=list, item_dtype=int)
    [1, 2, 3]

    Parsing an environment variable as a tuple of integers:

    >>> os.environ["ENV_VAR"] = "1,2,3"
    >>> enve.get("ENV_VAR", dtype=tuple, item_dtype=int)
    (1, 2, 3)
    """  # noqa: E501
    dtype = str if dtype is None else dtype
    accepted_dtypes = (bool, bytes, float, int, str, list, tuple)
    if dtype not in accepted_dtypes:
        err_msg = f"Invalid type '{dtype}' for '{envvar}'. Expected one of {accepted_dtypes}."
        raise TypeError(err_msg)

    if default is not UNSET and default_factory is not None:
        err_msg = (
            f"Cannot use both 'default' and 'default_factory' for '{envvar}'. Use one or the other."
        )
        raise EnvError(err_msg)

    if default not in (UNSET, None):
        if not isinstance(default, dtype):
            err_msg = (
                f"Default value '{default!r}' (type={type(default).__name__}) for '{envvar}'"
                f" is not of type '{dtype.__name__}'."
            )
            raise TypeError(err_msg)

        if dtype in (list,):
            warn_msg = (
                f"Using a default value for '{envvar}' of type '{dtype.__name__}'"
                " is not recommended as it may be mutated."
            )
            warnings.warn(warn_msg, UserWarning, stacklevel=2)

    if default_factory is not None and not callable(default_factory):
        err_msg = f"Default factory '{default_factory}' for '{envvar}' is not callable."
        raise TypeError(err_msg)

    value = _get_envvar_value(envvar, docker_secret=docker_secret)
    if value is UNSET:
        if not isinstance(default, Sentinel):
            return default
        if default_factory is not None:
            return default_factory()

        err_msg = (
            f"Environment variable '{envvar}' is not set and no default"
            " or default_factory is provided."
        )
        raise EnvError(err_msg)

    value = cast(str, value)

    if deprecated:
        if not deprecated_msg:
            deprecated_msg = f"The '{envvar}' environment variable is deprecated."
        warnings.warn(deprecated_msg, DeprecationWarning, stacklevel=2)

    if dtype is bool:
        return _parse_bool(envvar, value)
    elif dtype is bytes:
        return value.encode("utf-8")
    elif dtype is float:
        return _parse_float(envvar, value)
    elif dtype is int:
        return _parse_int(envvar, value)
    elif dtype is list:
        values = value.split(item_sep)
        if item_dtype is None:
            return values
        values = _parse_list(envvar, values, item_dtype)
        return cast(list[ItemType], values)
    elif dtype is tuple:
        values = value.split(item_sep)
        if item_dtype is None:
            return tuple(values)
        values = _parse_list(envvar, values, item_dtype)
        return cast(tuple[ItemType, ...], tuple(values))
    else:  # dtype is str
        return value



"""CLI entrypoint for the enve library."""

import argparse
import json
import os
import sys

from collections.abc import Sequence

from enve.parser import UNSET, get


def _get_docker_secret_value(docker_secret: str | None) -> bool | str:
    """Parse the docker secret value from the CLI."""
    if docker_secret is None:
        return False
    if not docker_secret:
        return True

    return docker_secret


def main(sys_args: Sequence[str] | None = None) -> None:
    """CLI entrypoint for the enve library."""
    from enve import __version__

    prog = "enve"
    parser = argparse.ArgumentParser(
        prog=prog,
        description="Print environment variables values without worrying about substitutions.",
    )
    parser.add_argument(
        "env_var",
        help=(
            "The environment variable to print the value of."
            " If unset, print all environment variables."
        ),
        nargs="?",
    )

    parser_grp = parser.add_argument_group("Parser options")
    parser_grp.add_argument(
        "-d",
        "--default",
        action="store",
        const="",
        help=(
            "The default value to use if the environment variable is not set."
            " This can be used a boolean flag (i.e., this option can be set"
            " without a value). In this case, the default value will be"
            " an empty string if the environment variable is not set."
        ),
        nargs="?",
    )
    parser_grp.add_argument(
        "-s",
        "--docker-secret",
        action="store",
        const="",
        help=(
            "If set, specify that the environment variable can also be a Docker secret."
            " This can be used a boolean flag (i.e., this option can be set without a value)."
            " If set this way, a docker secret with the same name as the environment will"
            " be search for. If set with a value, the value will be used as the docker secret name."
        ),
        nargs="?",
    )

    global_env_grp = parser.add_argument_group("Global environment options")
    global_env_grp.add_argument(
        "-0",
        "--null",
        action="store_true",
        help=(
            "End each output line with a null character instead of a newline"
            " when printing all environment variables."
        ),
    )
    global_env_grp.add_argument(
        "-j",
        "--json",
        action="store_true",
        help=(
            "Output the environment variable as a JSON object."
            " This is only used when printing all environment variables."
        ),
    )

    parser.add_argument("--version", action="version", version=f"{prog} {__version__}")
    parser._positionals.title = "Positional arguments"
    parser._optionals.title = "Optional arguments"

    args = parser.parse_args(args=sys_args)
    if not args.env_var and not sys.stdin.isatty():
        args.env_var = sys.stdin.read().strip()

    if not args.env_var:
        eol = "\0" if args.null else "\n"

        if args.json:
            output_str = json.dumps(dict(os.environ))
        else:
            output_str = eol.join(f"{k}={v}" for k, v in os.environ.items())

        sys.stdout.write(f"{output_str}{eol}")
        parser.exit(status=0)

    env_var = str(args.env_var)
    default = UNSET if args.default is None else str(args.default)
    docker_secret = _get_docker_secret_value(str(args.docker_secret))

    try:
        try:
            env_value = get(env_var, default=default, docker_secret=docker_secret)
            sys.stdout.write(env_value)
            parser.exit(status=0)
        except ValueError as err:
            parser.exit(status=1, message=f"{err}\n")
    except BrokenPipeError:  # pragma: no cover
        # NOTE: This is tested but I can't figure out how to make it covered.
        # Python flushes standard streams on exit; redirect remaining output
        # to devnull to avoid another BrokenPipeError at shutdown
        devnull = os.open(os.devnull, os.O_WRONLY)
        os.dup2(devnull, sys.stdout.fileno())
        sys.exit(1)  #  Python exits with error code 1 on EPIPE


if __name__ == "__main__":
    main()



# MIT License

# Copyright (c) 2025 Aurélien Chick

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
"""Environment variable parser tests."""

import re

from pathlib import Path
from typing import Any

import pytest

from pyfakefs.fake_filesystem import FakeFilesystem

import enve


ENVVARS = {
    "ENVVAR": ("foo", None),
    "ENVVAR_BYTES": (b"foo", bytes),
    "ENVVAR_INT": (42, int),
    "ENVVAR_FLOAT": (3.14, float),
    "ENVVAR_STR": ("bar", str),
}


@pytest.mark.parametrize("envvar", list(ENVVARS.keys()))
def test_get_envvar(monkeypatch: pytest.MonkeyPatch, envvar: str) -> None:
    """Test parsing environment variables."""
    expected_value, dtype = ENVVARS[envvar]
    expected_type = str if dtype is None else dtype

    str_val = expected_value.decode() if isinstance(expected_value, bytes) else str(expected_value)
    monkeypatch.setenv(envvar, str_val)

    result = enve.get(envvar, dtype=dtype)
    assert isinstance(result, expected_type)
    assert result == expected_value


@pytest.mark.parametrize("value", ["1", "on", "T", "True", "t", "true", "y", "yes"])
def test_get_envvar_bool_truthy(monkeypatch: pytest.MonkeyPatch, value: str) -> None:
    """Test parsing truthy boolean values."""
    monkeypatch.setenv("ENVVAR_BOOL", value)

    result = enve.get("ENVVAR_BOOL", dtype=bool)
    assert isinstance(result, bool)
    assert result is True


@pytest.mark.parametrize("value", ["", "0", "off", "F", "False", "f", "false", "n", "no"])
def test_get_envvar_bool_falsy(monkeypatch: pytest.MonkeyPatch, value: str) -> None:
    """Test parsing falsy boolean values."""
    monkeypatch.setenv("ENVVAR_BOOL", value)

    result = enve.get("ENVVAR_BOOL", dtype=bool)
    assert isinstance(result, bool)
    assert result is False


@pytest.mark.parametrize("default", [True, False])
def test_get_envvar_bool_missing_with_default(
    monkeypatch: pytest.MonkeyPatch, *, default: bool
) -> None:
    """Test parsing missing boolean values with default."""
    monkeypatch.delenv("ENVVAR_BOOL", raising=False)

    result = enve.get("ENVVAR_BOOL", default=default, dtype=bool)
    assert isinstance(result, bool)
    assert result is default


@pytest.mark.parametrize(
    ("dtype", "factory", "expected_value"),
    [(str, lambda: "foo", "foo"), (list, lambda: ["foo"], ["foo"])],
)
def test_get_envvar_with_default_factory(
    monkeypatch: pytest.MonkeyPatch, dtype: Any, factory: Any, expected_value: Any
) -> None:
    """Test parsing environment variables with a default factory."""
    monkeypatch.delenv("ENVVAR", raising=False)

    result = enve.get("ENVVAR", dtype=dtype, default_factory=factory)
    assert result == expected_value


@pytest.mark.parametrize("dtype", [list, tuple])
@pytest.mark.parametrize(
    ("str_value", "item_dtype", "expected_value"),
    [
        ("1,2,3", None, ["1", "2", "3"]),
        ("1,2,3", str, ["1", "2", "3"]),
        ("1,2,3", int, [1, 2, 3]),
        ("1.0,2.0,3.0", float, [1.0, 2.0, 3.0]),
        ("true,false", bool, [True, False]),
        ("foo", bytes, [b"foo"]),
    ],
)
def test_get_envvar_container_type(
    monkeypatch: pytest.MonkeyPatch,
    dtype: type[list | tuple],
    str_value: str,
    item_dtype: Any,
    expected_value: Any,
) -> None:
    """Test parsing list and tuple environment variables."""
    monkeypatch.setenv("ENVVAR_LIST", str_value)

    result = enve.get("ENVVAR_LIST", dtype=dtype, item_dtype=item_dtype)
    assert isinstance(result, dtype)
    assert result == dtype(expected_value)


@pytest.mark.parametrize("dtype", [list, tuple])
def test_get_envvar_container_with_custom_item_sep(
    monkeypatch: pytest.MonkeyPatch, dtype: type[list | tuple]
) -> None:
    """Test parsing list/tuple environment variables with a custom item separator."""
    monkeypatch.setenv("ENVVAR_LIST", "1;2;3")

    result = enve.get("ENVVAR_LIST", dtype=dtype, item_dtype=int, item_sep=";")
    assert isinstance(result, dtype)
    assert result == dtype([1, 2, 3])


@pytest.mark.parametrize("dtype", [list, tuple])
def test_get_envvar_container_with_invalid_item_dtype(
    monkeypatch: pytest.MonkeyPatch, dtype: type[list | tuple]
) -> None:
    """Test that an invalid item dtype raises a TypeError."""
    invalid_item_dtype = frozenset
    expected_msg = (
        f"Invalid item type '{invalid_item_dtype}' for 'ENVVAR_LIST'."
        f" Expected one of ({bool}, {bytes}, {float}, {int}, {str})."
    )

    monkeypatch.setenv("ENVVAR_LIST", "1,2,3")
    with pytest.raises(TypeError, match=re.escape(expected_msg)):
        enve.get("ENVVAR_LIST", dtype=dtype, item_dtype=invalid_item_dtype)


@pytest.mark.parametrize("secret", ["ENVVAR", "envvar"])
def test_get_envvar_from_docker_secrets(
    monkeypatch: pytest.MonkeyPatch, fs: FakeFilesystem, secret: str
) -> None:
    """Test that getting an envvar from Docker secrets works."""
    monkeypatch.delenv("ENVVAR", raising=False)
    fs.create_file(f"/run/secrets/{secret}", contents="foo")

    result = enve.get("ENVVAR", docker_secret=True)
    assert isinstance(result, str)
    assert result == "foo"


def test_get_envvar_from_non_existing_docker_secrets_with_default(
    monkeypatch: pytest.MonkeyPatch,
) -> None:
    """Test that getting an envvar from Docker secrets works."""
    monkeypatch.delenv("ENVVAR", raising=False)
    monkeypatch.setattr(Path, "exists", lambda _: False)

    result = enve.get("ENVVAR", default="foo", docker_secret=True)
    assert isinstance(result, str)
    assert result == "foo"


def test_get_envvar_from_named_docker_secrets(
    monkeypatch: pytest.MonkeyPatch, fs: FakeFilesystem
) -> None:
    """Test that getting an envvar from Docker secrets works."""
    monkeypatch.delenv("ENVVAR", raising=False)
    fs.create_file(f"/run/secrets/foobar", contents="foo")

    result = enve.get("ENVVAR", docker_secret="foobar")
    assert isinstance(result, str)
    assert result == "foo"


def test_get_envvar_from_non_existing_named_docker_secrets_with_default(
    monkeypatch: pytest.MonkeyPatch,
) -> None:
    """Test that getting an envvar from Docker secrets works."""
    monkeypatch.delenv("ENVVAR", raising=False)
    monkeypatch.setattr(Path, "exists", lambda _: False)

    result = enve.get("ENVVAR", default="foo", docker_secret="barino")
    assert isinstance(result, str)
    assert result == "foo"


def test_get_envvar_with_invalid_dtype(monkeypatch: pytest.MonkeyPatch) -> None:
    """Test that an invalid dtype raises a TypeError."""
    invalid_dtype = frozenset
    expected_msg = (
        f"Invalid type '{invalid_dtype}' for 'ENVVAR'."
        f" Expected one of ({bool}, {bytes}, {float}, {int}, {str}, {list}, {tuple})."
    )

    monkeypatch.setenv("ENVVAR", "foo")
    with pytest.raises(TypeError, match=re.escape(expected_msg)):
        enve.get("ENVVAR", dtype=invalid_dtype)


@pytest.mark.parametrize("dtype", [None, str, int, float, bool])
def test_get_non_existing_envvar_with_none_default(
    monkeypatch: pytest.MonkeyPatch,
    dtype: type[str | int | float | bool] | None,
) -> None:
    """Test that a None default returns None."""
    monkeypatch.setenv("ENVVAR", "foo")

    result = enve.get("ENVVAR_non_existing", default=None, dtype=dtype)
    assert result is None


def test_get_non_existing_envvar_with_no_default_raises(monkeypatch: pytest.MonkeyPatch) -> None:
    """Test that a non-existing envvar with no default raises a ValueError."""
    expected_msg = (
        "Environment variable 'ENVVAR' is not set and no default or default_factory is provided."
    )

    monkeypatch.delenv("ENVVAR", raising=False)
    with pytest.raises(ValueError, match=re.escape(expected_msg)):
        enve.get("ENVVAR")


def test_get_envvar_with_invalid_default_type(monkeypatch: pytest.MonkeyPatch) -> None:
    """Test that getting an envvar with an invalid default type raises a TypeError."""
    invalid_default = 42
    expected_msg = "Default value '42' (type=int) for 'ENVVAR' is not of type 'str'."

    monkeypatch.setenv("ENVVAR", "foo")
    with pytest.raises(TypeError, match=re.escape(expected_msg)):
        enve.get("ENVVAR", default=invalid_default, dtype=str)


def test_get_envvar_with_default_and_default_factory_raises(
    monkeypatch: pytest.MonkeyPatch,
) -> None:
    """Test that getting an envvar with both default and default_factory raises a ValueError."""
    expected_msg = (
        "Cannot use both 'default' and 'default_factory' for 'ENVVAR'. Use one or the other."
    )

    monkeypatch.setenv("ENVVAR", "foo")
    with pytest.raises(ValueError, match=re.escape(expected_msg)):
        enve.get("ENVVAR", default="bar", default_factory=lambda: "baz")


def test_get_envvar_with_list_default_raises_warning(monkeypatch: pytest.MonkeyPatch) -> None:
    """Test that getting an envvar with a list default raises a warning."""
    expected_msg = (
        "Using a default value for 'ENVVAR' of type 'list' is not recommended as it may be mutated."
    )

    monkeypatch.delenv("ENVVAR", raising=False)
    with pytest.warns(UserWarning, match=re.escape(expected_msg)):
        value = enve.get("ENVVAR", default=["bar"], dtype=list)

    assert value == ["bar"]


def test_get_envvar_with_invalid_default_factory_raises(monkeypatch: pytest.MonkeyPatch) -> None:
    """Test that getting an envvar with an invalid default factory raises a TypeError."""
    invalid_default_factory = 42
    expected_msg = "Default factory '42' for 'ENVVAR' is not callable."

    monkeypatch.setenv("ENVVAR", "1,2,3")
    with pytest.raises(TypeError, match=re.escape(expected_msg)):
        enve.get("ENVVAR", default_factory=invalid_default_factory, dtype=list)


def test_get_deprecated_envvar_with_custom_msg(monkeypatch: pytest.MonkeyPatch) -> None:
    """Test that a deprecated envvar raises a DeprecationWarning."""
    msg = "This envvar is deprecated!"

    monkeypatch.setenv("ENVVAR", "foo")
    with pytest.deprecated_call(match=re.escape(msg)):
        value = enve.get("ENVVAR", deprecated=True, deprecated_msg=msg)

    assert value == ENVVARS["ENVVAR"][0]


def test_get_deprecated_envvar_with_default_msg(monkeypatch: pytest.MonkeyPatch) -> None:
    """Test that a deprecated envvar raises a DeprecationWarning."""
    expected_msg = "The 'ENVVAR' environment variable is deprecated."

    monkeypatch.setenv("ENVVAR", "foo")
    with pytest.deprecated_call(match=re.escape(expected_msg)):
        value = enve.get("ENVVAR", deprecated=True)

    assert value == ENVVARS["ENVVAR"][0]


def test_get_invalid_int_envvar(monkeypatch: pytest.MonkeyPatch) -> None:
    """Test that an invalid int envvar raises a ValueError."""
    expected_msg = "Environment variable 'ENVVAR_INT' is not a valid integer: 'foo'"

    monkeypatch.setenv("ENVVAR_INT", "foo")
    with pytest.raises(ValueError, match=re.escape(expected_msg)):
        enve.get("ENVVAR_INT", dtype=int)


def test_get_invalid_float_envvar(monkeypatch: pytest.MonkeyPatch) -> None:
    """Test that an invalid float envvar raises a ValueError."""
    expected_msg = "Environment variable 'ENVVAR_FLOAT' is not a valid float: 'foo'"

    monkeypatch.setenv("ENVVAR_FLOAT", "foo")
    with pytest.raises(ValueError, match=re.escape(expected_msg)):
        enve.get("ENVVAR_FLOAT", dtype=float)


def test_get_invalid_bool_envvar(monkeypatch: pytest.MonkeyPatch) -> None:
    """Test that an invalid bool envvar raises a ValueError."""
    expected_msg = (
        "Environment variable 'ENVVAR_BOOL' is not a valid boolean value. "
        "Expected one of ('1', 'on', 't', 'true', 'y', 'yes',"
        " '', '0', 'off', 'f', 'false', 'n', 'no')."
    )

    monkeypatch.setenv("ENVVAR_BOOL", "foo")
    with pytest.raises(ValueError, match=re.escape(expected_msg)):
        enve.get("ENVVAR_BOOL", dtype=bool)
